/**
 * Die Klasse <code>Abwehr</code> erbt von der Klasse <code>Spieler</code>.
 * In der Klasse <code>Abwehr</code> steht, wie sich die Abwehr in bestimmten Situationen Verhalten soll.
 * Sie ist also die Intelligenz bzw. KI der Abwehr.
 *
 * @version <br>3.0 Copyright 2002 by Team 18<br><br>
 * @author
 * <ul>
 * <li><a href="mailto:matze@gmx.net">Matthias Kunze</a></li>
 * <li><a href="mailto:holger.roessig@gmx.de">Holger Rssig</a></li>
 * <li><a href="mailto:bstrauss@gmx.de">Bastian Strauss</a></li>
 * </ul>
 * @since <br>JDK v 1.3.1<br><br>
 * @see <br> Communication.java
 * @see Weltmodell.java <br><br>
 */

public class Abwehr extends Spieler
{
    private double xStart, yStart;        //Position des Spielers vor dem Anpfiff
    private int x=0;

    /**
     * Konstruktor von <code>Abwehr</code>
     *
     * @param teamname    Teamname
     * @param servername  Name des Servers
     * @param serverport  Port des Servers
     * @param xStart      X-Startkoordinate
     * @param yStart      Y-Startkoordinate
     */
    public Abwehr(String teamname, String servername, int serverport, double xStart, double yStart)
    {
        this.xStart = xStart;
        this.yStart = yStart;
        this.teamname = teamname;
        this.communication = new Communication(servername, serverport);
        this.weltmodell = new Weltmodell(communication);
        communication.weltmodell = weltmodell;
        weltmodell.xStart = xStart;
        weltmodell.yStart = yStart;
        weltmodell.teamname = teamname;
        weltmodell.Sollpos.setXAbs(xStart);
        weltmodell.Sollpos.setYAbs(yStart);
    }

    /**
     * Startet den Thread der <code>Abwehr</code>
     *
     */
    public void run()
    {
        init(teamname);
        weltmodell.sensor.receive();
        status = CONNECTED;
        play();
    }

    /**
     * Taktik der <code>Abwehr</code>
     *
     */
    private void play()
    {
    while(true)
        {
            switch (status) {
            case NOT_CONNECTED:
                init(teamname);
                status = CONNECTED;
                break;
            case CONNECTED:
                move(xStart, yStart);   // Spielrichtung von Links nach Rechts         
                if (weltmodell.side==1) // Spielrichtung von Rechts nach Links
                    turn(180);
                weltmodell.sensor.receive();
                status = PLAYING;
                break;
            case PLAYING:
                while(weltmodell.seezeitpunkt==false){weltmodell.sensor.receive();}
                weltmodell.seezeitpunkt=false;   

                if (weltmodell.NormalPlay)  // Das Spiel laeuft
                {
                    // Wenn Ball nicht mehr aktuell ist suche die neue Ballposition
                    if (weltmodell.time > (weltmodell.ball.getTime()+5)){
                        turn(90);
                    }
                    
                    // Ball in Reichweite ==> Ball zum Mittelfeld passen
                    else if (weltmodell.ball.getDistance() < reichweite)
                    {
                        if (weltmodell.yAktAbs>0) {pass(0,15,100);}
                        else {pass(0,-15,100);}
                        reichweite = defaultReichweite;
                    }
                       
                    // Wenn Ball im Zustaendigkeitsbereich, dann gehe zum Ball
                    else if (ballinarea(18,14) || weltmodell.ball.getDistance() < 10)
                    {
                        if (weltmodell.ball.getDistance() > 2){
                            reichweite = 1.9;
                            geheZuBall(100);
                        }

                        else if (weltmodell.ball.getDistance() > 1){
                            reichweite = defaultReichweite;
                            geheZuBall(65);
                        }

                        else
                            geheZuBall(40);
                    }
                    
                    // Gehe zu Sollposition
                    else
                    {
                        if (weltmodell.ball.getXAbs() > 0)
                        {
                        if (calcDistance(weltmodell.Sollpos.getXAbs() + 25, weltmodell.Sollpos.getYAbs()) > 10)
                            geheZu(weltmodell.Sollpos.getXAbs() + 25, weltmodell.Sollpos.getYAbs(),100);
                        else if (calcDistance(weltmodell.Sollpos.getXAbs() + 25, weltmodell.Sollpos.getYAbs()) > 2)
                            geheZu(weltmodell.Sollpos.getXAbs() + 25, weltmodell.Sollpos.getYAbs(),20);    
                        }
                        else 
                        {
                        if (calcDistance(weltmodell.Sollpos) > 10)
                            geheZu(weltmodell.Sollpos,100);
                        else if (calcDistance(weltmodell.Sollpos) > 2)
                            geheZu(weltmodell.Sollpos,20);
                        }
                    }
                }
                
                else   // Das Spiel ist unterbrochen
                {

                    // Der Gegner hat ein Tor erzielt
                    if(weltmodell.opponentGoal)
                    {
                        move(xStart, yStart);
                        if (weltmodell.side==1)
                            turn(180);
                        weltmodell.opponentGoal = false;
                    }

                    // Wir haben ein Tor erzielt
                    if (weltmodell.ourGoal)
                    {
                        if (weltmodell.side==0)
                        {
                            if(yStart==-24)
                                move(-20,-10);   
                            else if(yStart==-0.2)
                                move(-40,-10); 
                            else if(yStart==24)
                                move(-37,12); 
                            else if(yStart==0.2)
                                move(-23,12); 
                        }
                        else
                            {
                            if(yStart==-24)
                                move(-20,10);   
                            else if(yStart==-0.2)
                                move(-40,10); 
                            else if(yStart==24)
                                move(-37,-12); 
                            else if(yStart==0.2)
                                move(-23,-12); 
                            }
                        for (int x=0; x<25; x++)turn(8);  
                        move(xStart, yStart);
                        if (weltmodell.side==1)
                            turn(180);
                        weltmodell.ourGoal = false;
                    }

                    // Wir haben Anstoss
                    if(weltmodell.ourKickOff)
                    {
                        if (weltmodell.ball.getTime() == -1 || weltmodell.time > (weltmodell.ball.getTime()+2)){
                            turn(30);
                        }
                    }

                    // Der Gegner hat Anstoss
                    if(weltmodell.opponentKickOff)
                    {
                        if (weltmodell.ball.getTime() == -1 || weltmodell.time > (weltmodell.ball.getTime()+2)){
                            turn(30);
                        }
                    }

                    // Der Gegner bzw. unser Torwart hat Ballbesitz
                    if (weltmodell.opponentGoalieHasBall || weltmodell.ourGoalieHasBall
                        || weltmodell.opponentFreeKick || weltmodell.opponentKickIn
                        || weltmodell.opponentGoalKick || weltmodell.ourGoalKick)
                    {
                        if (weltmodell.ball.getXAbs() > 0)
                        {
                        if (calcDistance(weltmodell.Sollpos.getXAbs() + 25, weltmodell.Sollpos.getYAbs()) > 10)
                            geheZu(weltmodell.Sollpos.getXAbs() + 25, weltmodell.Sollpos.getYAbs(),100);
                        else if (calcDistance(weltmodell.Sollpos.getXAbs() + 25, weltmodell.Sollpos.getYAbs()) > 2)
                            geheZu(weltmodell.Sollpos.getXAbs() + 25, weltmodell.Sollpos.getYAbs(),20);    
                        }
                        else 
                        {
                        if (calcDistance(weltmodell.Sollpos) > 10)
                            geheZu(weltmodell.Sollpos,100);
                        else if (calcDistance(weltmodell.Sollpos) > 2)
                            geheZu(weltmodell.Sollpos,20);
                        }
                    }

                    // Wir haben Freistoss oder Einwurf
                    // gleiches Verhalten wie beim "normalen" Spiel
                    if (weltmodell.ourFreeKick || weltmodell.ourKickIn)
                    {
                        weltmodell.NormalPlay=true;    
                    }

                    // Der Gegner hat Eckball
                    if (weltmodell.opponentCornerKick)
                    {
                        if (calcDistance(weltmodell.Sollpos) > 10)
                            geheZu(weltmodell.Sollpos,100);
                        else if (calcDistance(weltmodell.Sollpos) > 2)
                            geheZu(weltmodell.Sollpos,20);
                    }

                    // Wir haben Eckball -> Eckball schieen, oder vorrcken
                    if (weltmodell.ourCornerKick)
                    {
                        if (weltmodell.ball.getXAbs() > 0)
                        {
                        if (calcDistance(weltmodell.Sollpos.getXAbs() + 35, weltmodell.Sollpos.getYAbs()) > 10)
                            geheZu(weltmodell.Sollpos.getXAbs() + 35, weltmodell.Sollpos.getYAbs(),100);
                        else if (calcDistance(weltmodell.Sollpos.getXAbs() + 35, weltmodell.Sollpos.getYAbs()) > 2)
                            geheZu(weltmodell.Sollpos.getXAbs() + 35, weltmodell.Sollpos.getYAbs(),20);    
                        }
                        else 
                        {
                        if (calcDistance(weltmodell.Sollpos) > 10)
                            geheZu(weltmodell.Sollpos,100);
                        else if (calcDistance(weltmodell.Sollpos) > 2)
                            geheZu(weltmodell.Sollpos,20);
                        }
                     }
                     
                    if (weltmodell.beforeKickOff){
                        move(xStart, yStart);    
                    }

                    }
                }
        }
    }
}
