/**
 * In der Klasse <code>AcousticEvent</code> werden alle Hear Nachrichten analysiert,
 * interpretiert und im Weltmodell abgespeichert.
 *
 * @version <br>3.0 Copyright 2002 by Team 18<br><br>
*
 * @author
 * <ul>
 * <li><a href="mailto:heiner.litz@blauton.com">Heiner Litz</a></li>
 * </ul>
 * @since <br>JDK v 1.3.1<br><br>
 * @see <br> Weltmodell.java <br><br>
 */

import java.util.StringTokenizer;

public class AcousticEvent extends Event
{
    private Weltmodell weltmodell;

    /**
     * Erstellt eine neue Instanz des Typs <code>AcousticEvent</code> und verwendet einen Tokanizer,
     * um die bergebene Nachricht zu unterteilen und die relevanten Informationen zu extrahieren.
     * Diese werden interpretiert und in den Variablen des Weltmodells gespeichert.
     *
     * @param hear message  Nachricht die der Spieler hrt
     * @param weltmodell  Weltmodell des Spielers
    */
    public AcousticEvent(String message, Weltmodell weltmodell)
    {
        this.weltmodell=weltmodell;

        // benutzt ein Leerzeichen als Begrenzungszeichen fr die Zeichenfolge
        StringTokenizer tokenizer = new StringTokenizer(message, " ");

        // wechselt zum nchsten Objekt
        tokenizer.nextToken();

        int time = Integer.parseInt(tokenizer.nextToken());

        // holt sich den Objekttyp
        String code=tokenizer.nextToken();

        if (!code.equals(" ")){

            //Die Hear Nachrichten der Trainer. Bleiben in dieser Version noch unbercksichtigt.
            if (code.equals("online_coach_left")) {}

            if (code.equals("online_coach_right")) {}

            //Die Hear Nachrichten des Schiedsrichters
            if (code.equals("referee"))
            {
                // Nachricht des Schiedsrichters auslesen
                String data = tokenizer.nextToken();

                //Normaler Spielablauf
                if(data.substring(0, 7).equals("play_on")){
                    weltmodell.NormalPlay=true;
                    weltmodell.opponentGoalieHasBall = false;  //Gegnerischer Torwart hat Ball aufgenommen
                    weltmodell.ourGoalieHasBall = false;
                    weltmodell.opponentGoal = false;      //Die Gegner haben ein Tor geschossen 50cycles true
                    weltmodell.ourGoal = false;
                    weltmodell.opponentFreeKick = false;  //Die Gegner haben Freistoss
                    weltmodell.ourFreeKick = false;
                    weltmodell.opponentOffside = false;   //Die Gegner bekommen in 30 cycles Freistoss wegen Abseits
                    weltmodell.ourOffside = false;
                    weltmodell.opponentCornerKick=false;  //Die Gegner haben Eckball
                    weltmodell.ourCornerKick = false;
                    weltmodell.opponentKickIn = false;    //Die Gegner haben Einwurf
                    weltmodell.ourKickIn = false;
                    weltmodell.opponentGoalKick = false;
                    weltmodell.ourGoalKick = false;
                    weltmodell.opponentKickOff = false;
                    weltmodell.ourKickOff = false;
                    weltmodell.halfTime = false;
                    weltmodell.beforeKickOff = false;
                }

                //Eine Mannschaft hat ein Tor erzielt.
                if(data.substring(0, 5).equals("goal_")){
                    if(data.charAt(5)=='l'){
                        if(weltmodell.side==0){weltmodell.ourGoal=true;}
                        else{weltmodell.opponentGoal=true;}
                    }
                    if(data.charAt(5)=='r'){
                        if(weltmodell.side==0){weltmodell.opponentGoal=true;}
                        else{weltmodell.ourGoal=true;}
                    }

                    weltmodell.NormalPlay=false;
                }

                //Eine Mannschaft hat Absto von der Torauslinie
                if(data.substring(0, 9).equals("goal_kick")){
                    if(data.charAt(10)=='l'){
                        if(weltmodell.side==0){weltmodell.ourGoalKick=true;}
                        else{weltmodell.opponentGoalKick=true;}
                    }
                    if(data.charAt(10)=='r'){
                        if(weltmodell.side==0){weltmodell.opponentGoalKick=true;}
                        else{weltmodell.ourGoalKick=true;}
                    }

                    weltmodell.NormalPlay=false;
                }

                //Eine Mannschaft hat Freistoss
                if(data.substring(0,9).equals("free_kick")){

                    if(data.charAt(10)=='l'){
                            if(weltmodell.side==0){weltmodell.ourFreeKick=true;}
                            else{weltmodell.opponentFreeKick=true;}}
                    if(data.charAt(10)=='r'){
                            if(weltmodell.side==0){weltmodell.opponentFreeKick=true;}
                            else{weltmodell.ourFreeKick=true;}
                    }
                    weltmodell.NormalPlay=false;
                }

                //Eine Mannschaft hat Eckstoss
                if(data.substring(0, 6).equals("corner")){
                    if(data.charAt(12)=='l'){
                            if(weltmodell.side==0){weltmodell.ourCornerKick=true;}
                            else{weltmodell.opponentCornerKick=true;}}
                    if(data.charAt(12)=='r'){
                            if(weltmodell.side==0){weltmodell.opponentCornerKick=true;}
                            else{weltmodell.ourCornerKick=true;}
                    }
                    weltmodell.NormalPlay=false;
                }

                //Ein Torwart hat den Ball in die Hnde genommen
                if(data.substring(0, 17).equals("goalie_catch_ball")){
                    if(data.charAt(18)=='l'){
                            if(weltmodell.side==0){weltmodell.ourGoalieHasBall=true;}
                            else{weltmodell.opponentGoalieHasBall=true;}}
                    if(data.charAt(18)=='r'){
                            if(weltmodell.side==0){weltmodell.opponentGoalieHasBall=true;}
                            else{weltmodell.ourGoalieHasBall=true;}
                    }
                    weltmodell.NormalPlay=false;
                }

                //Eine Mannschaft hat Abstoss vom Mittelpunkt
                if(data.substring(0, 8).equals("kick_off")){
                    if(data.charAt(9)=='l'){
                            if(weltmodell.side==0){weltmodell.ourKickOff=true;}
                            else{weltmodell.opponentKickOff=true;}}
                    if(data.charAt(9)=='r'){
                            if(weltmodell.side==0){weltmodell.opponentKickOff=true;}
                            else{weltmodell.ourKickOff=true;}
                    }
                    weltmodell.NormalPlay=false;
                }
                
                //Eine Mannschaft stand im Abseits
                if(data.substring(0, 7).equals("offside")){
                    weltmodell.NormalPlay=false;
                }
                
                //Eine Mannschaft hat Einwurf
                if(data.substring(0, 7).equals("kick_in")){
                    if(data.charAt(8)=='l'){
                            if(weltmodell.side==0){weltmodell.ourKickIn=true;}
                            else{weltmodell.opponentKickIn=true;}}
                    if(data.charAt(8)=='r'){
                            if(weltmodell.side==0){weltmodell.opponentKickIn=true;}
                            else{weltmodell.ourKickIn=true;}
                    }
                    weltmodell.NormalPlay=false;
                }
                
                //Halbzeit!
                if(data.substring(0, 9).equals("half_time")){
                    weltmodell.halfTime = true;
                }
                
                //Das Spiel befindet sich im Zustand before Kick_off
                if(data.substring(0, 15).equals("before_kick_off")){
                    weltmodell.NormalPlay=false;
                    weltmodell.beforeKickOff = true;
                }
                
            }
            // Spieler hren ihre eigenen Nachrichten. Bis jetzt nicht implementiert
            if (code.equals("self")){}

            // Spieler hoert eine Nachricht von einem anderen Spieler
            else {}

          }

    }
}