/**
 * In der Klasse <code>Communication</code> wird mit dem Server kommuniziert.
 *
 * @version <br>3.0 Copyright 2002 by Team 18<br><br>
 * @author
 * <ul>
 * <li><a href="mailto:florian_beier@web.de">Florian Beier</a></li>
 * <li><a href="mailto:matze@gmx.net">Matthias Kunze</a></li>
 * <li><a href="mailto:heiner.litz@blauton.com">Heiner Litz</a></li>
 * <li><a href="mailto:holger.roessig@gmx.de">Holger Rssig</a></li>
 * <li><a href="mailto:technische.informatik@gmx.de">Oliver Schuppe</a></li>
 * <li><a href="mailto:bstrauss@gmx.de">Bastian Strauss</a></li>
 * </ul>
 * @since <br>JDK v 1.3.1<br><br>
 * @see <br> Weltmodell.java <br><br>
 */


import java.io.*;
import java.net.*;

public class Communication
{
    private DatagramSocket socket;
    InetAddress serverAddress;
    private int serverport;
    public Weltmodell weltmodell;

    /**
     * Konstruktor von <code>Communication</code>
     *
     * @param servername  Name des Servers
     * @param serverport  Port des Servers
     *
    */
    public Communication(String servername, int serverport) {
        try {
            this.socket        = new DatagramSocket();
            this.serverAddress = InetAddress.getByName(servername);
            this.serverport    = serverport;
        }
        catch (SocketException se) {
            System.err.println("Communication: Could not create datagram socket.");
            System.exit(1);
        }
        catch (UnknownHostException uhe) {
            System.err.println("Communication: Could not determine the server's IP address.");
            System.exit(1);
        }
    }
    /**
     * Sendet eine Nachricht
     *
     * @param message <br>Nachricht die gesendet werden soll<br>
     *
     * @throws <br>"Communication: Could not send datagram packet."<br>
     */
    public void send(String message) {
        try {
            byte[] buffer = message.getBytes();
            DatagramPacket packet =
            new DatagramPacket(buffer, buffer.length, serverAddress, serverport);
            while (weltmodell.sendzeitpunkt == false) {weltmodell.sensor.receive();}
            socket.send(packet);
            weltmodell.sendzeitpunkt = false;
            //  TESTKOMMENTAR:
            //  System.out.println("GESENDET: " + message);
        }
        catch (IOException ioe) {
            System.err.println("Communication: Could not send datagram packet.");
        }
    }

    /**
     * Empfaengt Nachrichten
     *
     * @return <br>Nachricht die empfangen wurde<br>
     *
     * @throws <br>"Commmunication: Could not receive datagram packet."<br>
     */
    public String receive() {
        byte[] buffer = new byte[3024];
        DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
        try {
            socket.receive(packet);
        }
        catch (IOException ioe) {
            System.err.println("Commmunication: Could not receive datagram packet.");
        }
        serverport = packet.getPort();
        //  TESTKOMMENTAR:
        //  System.out.println("EMPFANGEN: " + new String(packet.getData()));
        return new String(packet.getData());
    }
}

