/**
 * In der Klasse <code>GraphikAuswahl</code> wird der Spieler ausgewaehlt,
 * dessen Weltmodell angezeigt werden soll.
 *
 * @version <br>3.0 Copyright 2002 by Team 18<br><br>
 * @author
 * <ul>
 * <li><a href="mailto:technische.informatik@gmx.de">Oliver Schuppe</a></li>
 * </ul>
 * @since <br>JDK v 1.3.1<br><br>
 * @see <br> GraphikDisplay.java
 * @see Weltmodell.java <br><br>
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class GraphikAuswahl extends JPanel implements Runnable{
    private Spieler[] mannschaft;
    //Radiobuttons fr alle Spieler
    private JRadioButton[] r =  {
        new JRadioButton("Spieler 1 (Torwart)"),
        new JRadioButton("Spieler 2 (Abwehr)"),
        new JRadioButton("Spieler 3 (Abwehr)"),
        new JRadioButton("Spieler 4 (Abwehr)"),
        new JRadioButton("Spieler 5 (Abwehr)"),
        new JRadioButton("Spieler 6 (Mittelfeld)"),
        new JRadioButton("Spieler 7 (Mittelfeld)"),
        new JRadioButton("Spieler 8 (Mittelfeld)"),
        new JRadioButton("Spieler 9 (Sturm)"),
        new JRadioButton("Spieler 10 (Sturm)"),
        new JRadioButton("Spieler 11 (Sturm)")
    };
    
    //Textfeld fr Statusmeldungen
    private JTextField sel = new JTextField(10);
    private JButton OKButton = new JButton("Graphik starten");
    
    //die eigentliche Graphik wird in einem eigenen Thread angezeigt
    public Thread anzeige;
    public Weltmodell aktuellesWeltmodell = new Weltmodell();
    private boolean doubleBuffer;

    private ButtonGroup bg = new ButtonGroup();

    /**
     * Konstruktor von <code>GraphikAuswahl</code>
     *
     * @param mannschaft  Array der Spieler
     */
    GraphikAuswahl(final Spieler[] mannschaft, final boolean doubleBuffer) {
        this.mannschaft = mannschaft;       
        this.doubleBuffer = doubleBuffer;

        ActionListener OK_lis = new ActionListener() {
            public void actionPerformed(ActionEvent e){
                for (int i = 0; i < r.length; i++) {
                    if (r[i].isSelected()) {
                        //gewhlte Auswahl ausgeben
                        System.out.println("Graphik wird fuer " + r[i].getText() + " gestartet...");
                        System.out.println("Bitte warten...");
                        //das Weltmodell des ausgewhlten Spielers
                        aktuellesWeltmodell = mannschaft[i].weltmodell;
                        anzeige = new Thread(new GraphikDisplay(aktuellesWeltmodell, doubleBuffer));
                        //Thread nur starten, falls dies noch nicht passiert ist.
                        if (!anzeige.isAlive()) anzeige.start();
                    }
                }
            }
        }; //Ende ActionListener OK Button *****************

        ItemListener lis = new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                //falls der Radiobutton ausgewhlt wurde...
                if (e.getStateChange() == ItemEvent.SELECTED)
                    for (int i = 0; i < r.length; i++) {
                        if (r[i].isSelected()) {
                            //...Auswahl in Statuszeile schreiben.
                            sel.setText(r[i].getText() + " selected");
                        }
                    }
            }
        }; //Ende ItemListener lis ***************************

        r[0].setSelected(true);
        sel.setEditable(false);
        //verschiedene Elemente im Fenster anordnen mit Layoutmanager
        setLayout(new GridLayout((r.length + 3), 1));
        add(new JLabel("Bitte einen Spieler ausw\u00e4hlen "));
        for (int i = 0; i < r.length; i++) {
            r[i].addItemListener(lis);
            add(r[i]);
            bg.add(r[i]);
        }
        OKButton.addActionListener(OK_lis);
        add(sel);
        add(OKButton);
    }  //Ende Konstuktor *********************************
    
    /**
    *  Startet die <code>GraphikAuswahl</code>
    *
    */
    public void run() {
        //neues Fenster mit Titel "Spieler whlen"
        JFrame f = new JFrame("Spieler whlen");
        //Graphikauswahl im Fenster ausgeben
        f.getContentPane().add(new GraphikAuswahl(mannschaft, doubleBuffer));
        //das Fenster auf die bentigte Gre bringen...
        f.pack();
        //...und sichtbar machen.
        f.setVisible(true);
    }
}
