/**
 * Die Klasse <code>Mittelfeld</code> erbt von der Klasse <code>Spieler</code>.
 * In der <code>Mittelfeld</code>-Klasse steht wie sich der Mittelfeldspieler in bestimmten Situationen Verhalten soll.
 * Sie ist also die Intelligenz bzw. KI der Mittelfeldspieler.
 *
 * @version <br>3.0 Copyright 2002 by Team 18<br><br>
 * @author
 * <ul>
 * <li><a href="mailto:matze@gmx.net">Matthias Kunze</a></li>
 * <li><a href="mailto:holger.roessig@gmx.de">Holger Rssig</a></li>
 * <li><a href="mailto:bstrauss@gmx.de">Bastian Strauss</a></li>
 * </ul>
 * @since <br>JDK v 1.3.1<br><br>
 * @see <br> Communication.java
 * @see Weltmodell.java <br><br>
 */

public class Mittelfeld extends Spieler
{
    private double xStart, yStart;        //Position des Spielers vor dem Anpfiff
    private int x=0;

    /**
     * Konstruktor von <code>Mittelfeld</code>
     *
     * @param teamname    Teamname
     * @param servername  Name des Servers
     * @param serverport  Port des Servers
     * @param xStart      X-Startkoordinate
     * @param yStart      Y-Startkoordinate
     */
    public Mittelfeld(String teamname, String servername, int serverport, double xStart, double yStart)
    {
        this.xStart = xStart;
        this.yStart = yStart;
        this.teamname = teamname;
        this.communication = new Communication(servername, serverport);
        this.weltmodell = new Weltmodell(communication);
        communication.weltmodell = weltmodell;
        weltmodell.xStart = xStart;
        weltmodell.yStart = yStart;
        weltmodell.teamname = teamname;
        weltmodell.Sollpos.setXAbs(xStart+15);
        weltmodell.Sollpos.setYAbs(yStart);
    }

    /**
     * Startet den <code>Mittelfeldspieler</code>
     *
     */
    public void run()
    {
        init(teamname);
        weltmodell.sensor.receive();
        status = CONNECTED;
        play();
    }

    /**
     * Die Taktik des <code>Mittelfeldspieler</code>'s
     *
     */
    private void play()
    {
    while(true)
        {
            switch (status) {
            case NOT_CONNECTED:
                init(teamname);
                status = CONNECTED;
                break;
            case CONNECTED:
                move(xStart, yStart*0.9);
                if (weltmodell.side==1)
                    turn(180);
                weltmodell.sensor.receive();
                status = PLAYING;
                break;
            case PLAYING:
                while(weltmodell.seezeitpunkt==false){weltmodell.sensor.receive();}
                weltmodell.seezeitpunkt=false;   

                if (weltmodell.NormalPlay)  // Das Spiel laeuft
                {
                    // Wenn die Ballinformation zu alt ist; Ball suchen
                    if (weltmodell.time > (weltmodell.ball.getTime()+5)){
                        turn(90);
                    }
                    
                    // Wenn nah genug am Ball; etwas mit dem Ball machen
                    else if (weltmodell.ball.getDistance() < reichweite)
                    {
                        reichweite = defaultReichweite;
                         // Nah genug am Tor; dann aufs Tor schiessen
                         if ((weltmodell.goalR.getDistance() < 30))
                             goalkick(100);
                         // Nicht nah genug am Tor; Passsempfaenger suchen
                         else
                         {                                             
                            ObjectPerception PassEmpfaenger = PassEmpfaenger();
                            // Wenn kein Passempfaenger gefunden, Ball nach vorne bolzen
                            if (PassEmpfaenger == null)
                                goalkick(100);
                            // Wenn Passempfaenger gefunden, zu ihm passen
                            else{
                                pass(PassEmpfaenger,calcDistance(PassEmpfaenger) *3 );  
                            }     
                         }
                     }
                    // Wenn der Ball im Zustaendigkeitsbereich ist, zu ihm hinlaufen
                    else if (ballinarea(18,12) || weltmodell.ball.getDistance() < 10)
                    {
                         if (weltmodell.ball.getDistance() > 2){
                             reichweite = 1.9;
                             geheZuBall(100);
                         }
                         else if (weltmodell.ball.getDistance() > 1){
                             reichweite = defaultReichweite;     
                             geheZuBall(65);
                         }
                         else
                             geheZuBall(40);
                    }
                    // Wenn der Ball nicht im Zustaendigkeitsbereich ist zur Sollposition laufen
                    else
                    {
                        if (calcDistance(weltmodell.Sollpos) > 10)
                             geheZu(weltmodell.Sollpos,100);
                         else if (calcDistance(weltmodell.Sollpos) > 2)
                            geheZu(weltmodell.Sollpos,20);
                     }
                }
                
                // Das Spiel ist unterbrochen
                else
                {
                    // Der Gegner hat ein Tor erzielt
                    if(weltmodell.opponentGoal)
                    {
                        move(xStart, yStart*0.9);
                        if (weltmodell.side==1)
                            turn(180);
                        weltmodell.opponentGoal = false;
                    }

                    // Wir haben ein Tor erzielt
                    if (weltmodell.ourGoal)
                    {
                    if (weltmodell.side==0)
                        {
                        if(yStart==-24)
                            move(-36,13);   
                        else if(yStart==0)
                            move(-33,14); 
                        else if(yStart==24)
                            move(-31,15);
                        }
                    else
                        {    
                        if(yStart==-24)
                            move(-36,-13);   
                        else if(yStart==0)
                            move(-33,-14); 
                        else if(yStart==24)
                            move(-31,-15);
                        }
                             
                        for (int x=0; x<25; x++)
                            turn(8);                       
                        move(xStart, yStart*0.9);
                        weltmodell.ourGoal = false;
                    }

                    // Wir haben Anstoss
                    if(weltmodell.ourKickOff)
                    {
                        if (weltmodell.ball.getTime() == -1 || weltmodell.time > (weltmodell.ball.getTime()+2)){
                            turn(30);
                        }
                    }

                    // Der Gegner hat Anstoss
                    if(weltmodell.opponentKickOff)
                    {
                        if (weltmodell.ball.getTime() == -1 || weltmodell.time > (weltmodell.ball.getTime()+2)){
                            turn(30);
                        }
                    }

                    // Der Gegner bzw. unser Torwart hat Ballbesitz
                    if (weltmodell.opponentGoalieHasBall || weltmodell.ourGoalieHasBall
                        || weltmodell.opponentFreeKick || weltmodell.opponentKickIn
                        || weltmodell.opponentGoalKick || weltmodell.ourGoalKick)
                    {
                         if (calcDistance(weltmodell.Sollpos) > 10)
                             geheZu(weltmodell.Sollpos,100);
                         else if (calcDistance(weltmodell.Sollpos) > 2)
                            turn(weltmodell.ball.getDirection());
                    }

                    // Wir haben Freistoss oder Einwurf
                    // gleiches Verhalten wie beim "normalen" Spiel
                    if (weltmodell.ourFreeKick || weltmodell.ourKickIn)
                    {
                        weltmodell.NormalPlay=true;    
                    }

                    // Der Gegner hat Eckball
                    if (weltmodell.opponentCornerKick)
                    {
                         if (calcDistance(weltmodell.Sollpos) > 10)
                             geheZu(weltmodell.Sollpos,100);
                         else if (calcDistance(weltmodell.Sollpos) > 2)
                            turn(weltmodell.ball.getDirection());
                       
                    }

                    // Wir haben Eckball
                    if (weltmodell.ourCornerKick)
                    {
                             if (calcDistance(weltmodell.Sollpos) > 10)
                                 geheZu(weltmodell.Sollpos,100);
                             else if (calcDistance(weltmodell.Sollpos) > 2)
                                   geheZu(weltmodell.Sollpos,20);
                     }
                     
                    // Halbzeitpause. Nach dem Pausentee wieder auf die Startposition stellen. 
                    if (weltmodell.beforeKickOff){
                        move(xStart, yStart*0.9);
                    }                     

                    }
                }
        }
    }
}
