/**
 * In der Klasse <code>NPE</code> wird die Anzahl der bergebenen
 * Parameter <code>args[]</code> ausgewertet<br>und es werden die einzelnen
 * Spielertypen:
 * <ul>
 * <li><code>Abwehr</code></li>
 * <li><code>Stuermer</code>
 * <li><code>Torwart</code></li>
 * </ul> gestartet.
 * @param <br> args[0...3] bergebene Parameter aus der commandline
 * <ul>
 * <li>0 = Teamname</li>
 * <li>1 = Server-IP</li>
 * <li>2 = Server-Port</li>
 * <li>3 = GraphikAuswahl</li>
 * </ul>
 *
 * @throws <br> args.length nicht mit Syntax bereinstimmt:  <b>&lt</b>3 oder <b>&gt</b>4<br><br>
 * @version <br>2.x Copyright 2002 by Team 18<br><br>
 * @author
 * <ul>
 * <li><a href="mailto:florian_beier@web.de">Florian Beier</a></li>
 * <li><a href="mailto:matze@gmx.net">Matthias Kunze</a></li>
 * <li><a href="mailto:heiner.litz@blauton.com">Heiner Litz</a></li>
 * <li><a href="mailto:holger.roessig@gmx.de">Holger Rssig</a></li>
 * <li><a href="mailto:technische.informatik@gmx.de">Oliver Schuppe</a></li>
 * <li><a href="mailto:bstrauss@gmx.de">Bastian Strauss</a></li>
 * </ul>
 * @since <br>JDK v 1.3.1<br><br>
 * @see <br> Abwehr.java
 * @see GraphikAuswahl.java
 * @see Stuermer.java
 * @see Torwart.java <br><br>
 */

import java.lang.*;
import java.util.*;

public class NPE{

    public static void main (String[] args) {
        String host="localhost";
        int port=6000;
        String teamname="NPE";
        int i=0;
        boolean grafik=false;
        boolean doubleBuffer=false;
     
        while(i < args.length){
            if(args[i].equals("-t")){
               teamname = args[i+1];
               i += 2;
            }
            else if(args[i].equals("-p")){
                if (Character.isDigit(args[i+1].charAt(0))){
    	           port = Integer.valueOf(args[i + 1]).intValue();
    	           i += 2;
    	       }
    	       else i += 2;
            }
            else if(args[i].equals("-h")){
                host = args[i + 1];
    	        i += 2;
            }
            else if(args[i].equals("-g1")){
                doubleBuffer = true;
                grafik = true;
                i += 1;
            }
            else if(args[i].equals("-g2")){
                doubleBuffer = false;                
                grafik = true;
                i += 1;
            }
            else {i += 1;}
        }
        

        Torwart torwart = new Torwart(teamname, host, port, -50.5, 0);
        Abwehr abwehr1 = new Abwehr(teamname, host, port, -35, -24);
        Abwehr abwehr2 = new Abwehr(teamname, host, port, -30, -0.2);
        Abwehr abwehr3 = new Abwehr(teamname, host, port, -40, 0.2);
        Abwehr abwehr4 = new Abwehr(teamname, host, port, -35, 24);
        Mittelfeld mittelfeld1 = new Mittelfeld(teamname, host, port, -15, -24);
        Mittelfeld mittelfeld2 = new Mittelfeld(teamname, host, port, -15, 0);
        Mittelfeld mittelfeld3 = new Mittelfeld(teamname, host, port, -15, 24);
        Stuermer stuermer1 = new Stuermer(teamname, host, port, -0.4, -24);
        Stuermer stuermer2 = new Stuermer(teamname, host, port, -0.4, 0);
        Stuermer stuermer3 = new Stuermer(teamname, host, port, -0.4, 24);

        Spieler[] mannschaft = {torwart, abwehr1, abwehr2, abwehr3, abwehr4, mittelfeld1, mittelfeld2, mittelfeld3, stuermer1, stuermer2 , stuermer3};

        if(grafik){
            Thread graphik = new Thread(new GraphikAuswahl(mannschaft, doubleBuffer));
            graphik.start();
            //System.out.println("roboclient: mannschaft: " + mannschaft);
            //System.out.println("roboclient: mannschaft[0]: " + mannschaft[0]);            
        }
        
        
        //Spieler starten
        torwart.start();
        abwehr1.start();
        abwehr2.start();
        abwehr3.start();
        abwehr4.start();
        mittelfeld1.start();
        mittelfeld2.start();
        mittelfeld3.start();
        stuermer1.start();
        stuermer2.start();
        stuermer3.start();
    }
}

