/**
 * In der Klasse <code>RoboClient</code> wird die Anzahl der bergebenen
 * Parameter <code>args[]</code> ausgewertet<br>und es werden die einzelnen
 * Spielertypen:
 * <ul>
 * <li><code>Abwehr</code></li>
 * <li><code>Mittelfeld</code>
 * <li><code>Stuermer</code>
 * <li><code>Torwart</code></li>
 * </ul> gestartet.
 * @param <br> args[0...3] bergebene Parameter aus der commandline
 * <ul>
 * <li>0 = Teamname</li>
 * <li>1 = Server-IP</li>
 * <li>2 = Server-Port</li>
 * <li>3 = GraphikAuswahl (opt.):  1: Grafik mit DoubleBuffering; 2: Grafik ohne DoubleBuffering</li>
 * </ul>
 *
 * @version <br>3.0 Copyright 2002 by Team 18<br><br>
 * @author
 * <ul>
 * <li><a href="mailto:florian_beier@web.de">Florian Beier</a></li>
 * <li><a href="mailto:matze@gmx.net">Matthias Kunze</a></li>
 * <li><a href="mailto:heiner.litz@blauton.com">Heiner Litz</a></li>
 * <li><a href="mailto:holger.roessig@gmx.de">Holger Rssig</a></li>
 * <li><a href="mailto:technische.informatik@gmx.de">Oliver Schuppe</a></li>
 * <li><a href="mailto:bstrauss@gmx.de">Bastian Strauss</a></li>
 * </ul>
 * @since <br>JDK v 1.3.1<br><br>
 * @see <br> Abwehr.java
 * @see GraphikAuswahl.java
 * @see Stuermer.java
 * @see Torwart.java <br><br>
 */

public class RoboClient{

    public static void main (String[] args) {

        boolean doubleBuffer = false;

        if ((args.length<3) || (args.length > 4)){
            System.out.println("Eingabeformat: java RoboClient TEAMNAME SERVERNAME SERVERPORT GRAFIK (1/0)");
            System.exit(1);
        }

        Torwart torwart = new Torwart(args[0], args[1], Integer.parseInt(args[2]), -50.5, 0);
        Abwehr abwehr1 = new Abwehr(args[0], args[1], Integer.parseInt(args[2]), -35, -24);
        Abwehr abwehr2 = new Abwehr(args[0], args[1], Integer.parseInt(args[2]), -30, -0.2);
        Abwehr abwehr3 = new Abwehr(args[0], args[1], Integer.parseInt(args[2]), -40, 0.2);
        Abwehr abwehr4 = new Abwehr(args[0], args[1], Integer.parseInt(args[2]), -35, 24);
        Mittelfeld mittelfeld1 = new Mittelfeld(args[0], args[1], Integer.parseInt(args[2]), -15, -24);
        Mittelfeld mittelfeld2 = new Mittelfeld(args[0], args[1], Integer.parseInt(args[2]), -15, 0);
        Mittelfeld mittelfeld3 = new Mittelfeld(args[0], args[1], Integer.parseInt(args[2]), -15, 24);
        Stuermer stuermer1 = new Stuermer(args[0], args[1], Integer.parseInt(args[2]), -0.4, -24);
        Stuermer stuermer2 = new Stuermer(args[0], args[1], Integer.parseInt(args[2]), -0.4, 0);
        Stuermer stuermer3 = new Stuermer(args[0], args[1], Integer.parseInt(args[2]), -0.4, 24);

        Spieler[] mannschaft = {torwart, abwehr1, abwehr2, abwehr3, abwehr4, mittelfeld1, mittelfeld2, mittelfeld3, stuermer1, stuermer2 , stuermer3};

        //mit Grafik
        if (args.length == 4){
            if (Integer.parseInt(args[3]) == 1){doubleBuffer = true;}
            if (Integer.parseInt(args[3]) == 2){doubleBuffer = false;}
            if ((Integer.parseInt(args[3]) == 1) || (Integer.parseInt(args[3]) == 2)){
                Thread graphik = new Thread(new GraphikAuswahl(mannschaft, doubleBuffer));
                graphik.start();
            }
                else{
                    System.out.println("Eingabeformat: java RoboClient TEAMNAME SERVERNAME SERVERPORT GRAFIK (1/0)");
                    System.exit(1);
                }
        }

        //ohne Grafik
        if ((args.length == 3)||(args.length == 4)) {
            torwart.start();
            abwehr1.start();
            abwehr2.start();
            abwehr3.start();
            abwehr4.start();
            mittelfeld1.start();
            mittelfeld2.start();
            mittelfeld3.start();
            stuermer1.start();
            stuermer2.start();
            stuermer3.start();
        }
    }
}