/**
 * Die Klasse <code>Sensor</code> verarbeitet die eingehenden Nachrichten vom Server
 * welche dann an das entsprechende <code>Event</code>:
 * <ul>
 * <li><code>AcoustivEvent</code></li>
 * <li><code>BodyEvent</code>
 * <li><code>VisualEvent</code></li>
 * </ul> weiter gegeben werden.
 *
 * @author
 * <ul>
 * <li><a href="mailto:florian_beier@web.de">Florian Beier</a></li>
 * </ul>
 *
 * @version <br> 3.0 Copyright 2002 by Team 18<br><br>
 * @since <br>JDK v 1.3.1<br><br>
 * @see <br> AcousticEvent.java
 * @see BodyEvent.java
 * @see Communication.java
 * @see Event.java
 * @see VisualEvent.java
 * @see Weltmodell.java <br><br>
 *
*/

public class Sensor{
    private Weltmodell weltmodell;
    private Communication communication;
    public Event event;

    /**
     * Konstruktor von <code>Sensor</code>
     *
     * @param communication Kommunikation des Spielers 
     * @param weltmodell Weltmodell des Spielers
    */
    public Sensor(Communication communication, Weltmodell weltmodell){
        this.communication = communication;
        this.weltmodell = weltmodell;
    }

    /**
     * Verarbeitet die eingehenden Nachrichten vom Server
     * und leitet diese an die entsprechenden Events weiter
    */
    public void receive(){
        String msg;
        while (event == null) {
            
            //warte bis eine Nachricht vom Server kommt
            msg = communication.receive();

            //See-Nachricht wird an die Visual-Event weitergeleitet
            if (msg.startsWith("(see")){
                weltmodell.seezeitpunkt = true;
                weltmodell.anzahlSeeNachrichten += 1;
                event = new VisualEvent(msg, weltmodell);
            }

            //Body-Nachricht wird an das Body-Event weitergeleitet
            else if (msg.startsWith("(sense_body")){
                weltmodell.sendzeitpunkt = true;
                event = new BodyEvent(msg, weltmodell);
            }

            //Hear-Nachricht wird an die Acoustic-Event weitergeleitet
            else if (msg.startsWith("(hear")){event = new AcousticEvent(msg, weltmodell);}
            
            
            //Variable "side" wird gesetzt ja nach dem ob von links nach rechts
            //oder umgekehrt gespielt wird
            else if (msg.startsWith("(init l")){weltmodell.side=0;}
            else if (msg.startsWith("(init r")){weltmodell.side=1;}

            //Server und Player parameter werden nicht beachtet            
            else if (msg.startsWith("(player")){}
            else if (msg.startsWith("(server")){}
            
            //Errors werden ausgegeben
            else {/*System.out.println("EMPFANGS ERROR: "+msg);*/}
        }
        event = null;
    }
}