 /**
 * Die Klasse <code>Spieler</code> ist die Vaterklasse fr:
 * <ul>
 * <li><code>Abwehr</code></li>
 * <li><code>Mittelfeld</code></li>
 * <li><code>Stuermer</code></li>
 * <li><code>Torwart</code></li>
 * </ul>
 *
 *
 * @version <br>3.0 Copyright 2002 by Team 18<br><br>
 * @author
 * <ul>
 * <li><a href="mailto:matze@gmx.net">Matthias Kunze</a></li>
 * <li><a href="mailto:holger.roessig@gmx.de">Holger Rssig</a></li>
 * <li><a href="mailto:bstrauss@gmx.de">Bastian Strauss</a></li>
 * </ul>
 * @since <br>JDK v 1.3.1<br><br>
 * @see <br> Java.lang.Math.java
 * @see GraphikAuswahl.java
 * @see Stuermer.java
 * @see Torwart.java <br><br>
 */

import java.lang.Math;
import java.util.*;

abstract class Spieler extends Thread
{
    protected String teamname;

    final protected int NOT_CONNECTED = 0;
    final protected int CONNECTED     = 1;
    final protected int PLAYING       = 2;

    protected int status = NOT_CONNECTED;
    protected int time = -1;
    
    final protected double defaultReichweite = 1.0;    
    final protected double hoheReichweite = 2.0;
    final protected double extremeReichweite = 3.0;
    protected double reichweite = defaultReichweite;


    protected Communication communication;
    protected Weltmodell weltmodell;

        /**
         * Meldet den Spieler am Server an
         *
         * @param teamname Teamname
        */
        public void init(String teamname)
        {
            communication.send(new String("(init " + teamname + " (version 7.0))"));
        }

        //"Hhere" Funktionen:
        /**
         * Schiet den Ball aufs gegnerische Tor
         *
         * @param strength Die "Hrte" des Torschusses
         */
        public void goalkick(double strength)
        {
            double xGoal = 0;
            double yGoal = 0;
         
            xGoal = 52.5;
            yGoal = 0;

            double direction = calcDirection(xGoal, yGoal);
            kick(strength, direction);      //zur Position kicken
        }

        /**
         *
         * Kickt den Ball zu einer bestimmten Ziel-Position
         *
         * @param xPos X-Koordinate der Ziel-Position
         * @param yPos Y-Koordinate der Ziel-Position
         * @param p    Die "Hrte" des Passes
         */
        public void pass(double xPos, double yPos, double p)
        {
            ObjectPerception objekt = new ObjectPerception("xy-to-object", xPos, yPos);
            pass (objekt, p);
        }

        /**
         * Kickt den Ball zu einer bestimmten Ziel-Position
         *
         * @param objekt Ziel-Position vom Typ <code>ObjectPerception</code>
         * @param p      Die "Hrte" des Passes
         */
        public void pass(ObjectPerception objekt, double p)
        {
            double direction = calcDirection(objekt);
            kick(p, direction);
        }

        /**
         * Fhrt einen Laufimpuls (dash) des Spielers zu einer bestimmten Ziel-Position aus
         *
         *
         * @param objekt Ziel-Position vom Typ <code>ObjectPerception</code>
         * @param p      Die Strke des Laufimpulses
         */
        public void geheZu(ObjectPerception objekt, double p)
        {
            //Berechnung von turnangle (in Grad)
            double direction = calcDirection(objekt);
            if ((direction > 3 || direction < -3))
                {
                turn(direction);  //zur Position drehen (in Grad)
                }
            dash(p);              //zur Position laufen
        }

        /**
         * Fhrt einen Laufimpuls (dash) des Spielers zu einer bestimmten Ziel-Position aus
         *
         *
         * @param xPos  X-Koordinate der Ziel-Position
         * @param yPos  Y-Koordinate der Ziel-Position
         * @param p     Die Strke des Laufimpulses
         */
        public void geheZu(double xPos, double yPos, double p)
        {
            ObjectPerception objekt = new ObjectPerception("xy-to-object", xPos, yPos);
            geheZu (objekt, p);

        }

        /**
         * Luft zu einer bestimmten Ziel-Position bis der Abstand < 2
         * ohne sich dabei nach dem Ball zu orientieren
         *
         * @param xPos  X-Koordinate der Ziel-Position
         * @param yPos  Y-Koordinate der Ziel-Position
         * @param p     Die Strke des Laufimpulses
         */
        public void geheZuDirekt(double xPos, double yPos, double p)
        {
            ObjectPerception objekt = new ObjectPerception("xy-to-object", xPos, yPos);
            geheZuDirekt (objekt, p);

        }

        /**
         * Luft zu einer bestimmten Ziel-Position bis der Abstand < 2
         * ohne sich dabei nach dem Ball zu orientieren
         *
         * @param objekt Ziel-Position vom Typ <code>ObjectPerception</code>
         * @param p      Die Strke des Laufimpulses
         */
        public void geheZuDirekt(ObjectPerception objekt, double p)
        {
            double direction;
            while (calcDistance(objekt) > 2)
                {
                direction = calcDirection(objekt);
                turn(direction);        //zur Position drehen (in Grad)
                dash(p);                //zur Position laufen
                }
        }

        /**
         * Fhrt einen Laufimpuls (dash) des Spielers zu einer Position,
         * die in Bewegungsrichtung des Balls liegt, aus, um diesen abfangen
         * und kicken zu knnen
         *
         * @param p Die Strke des Laufimpulses
         */
        public void geheZuBall(double p)
        {
            double xalt = 0;        //alte X-Position des Balls
            double yalt = 0;        //alte Y-Position des Balls
            double xneu = 0;        //neue X-Position des Balls
            double yneu = 0;        //neue Y-Position des Balls
            double xball = 0;       //X-Koordinate des resultierenden Vektors
            double yball = 0;       //Y-Koordinate des resultierenden Vektors
            double strecke;         //Strecke die der Ball zwischen den 2 See-Nachrichten zurckgelegt hat
            int seezeit = weltmodell.anzahlSeeNachrichten;
            double z;               //Verlngerungsfaktor des Ballvektors
            double ZielPosX;        //X-Koordinate der Zielposition
            double ZielPosY;        //Y-Koordinate der Zielposition
            
                    xalt = weltmodell.ball.getXAbs();
                    yalt = weltmodell.ball.getYAbs();
                    
                    //Warten, bis neue Ballinformationen vorhanden sind      
                    while (weltmodell.anzahlSeeNachrichten == seezeit)
                        {
                            weltmodell.sensor.receive();
                        }
               
                    xneu = weltmodell.ball.getXAbs();
                    yneu = weltmodell.ball.getYAbs();
         
                //Ballvektor :
                xball = xneu - xalt;
                yball = yneu - yalt;
          
            //Strecke die der Ball zwischen den 2 See-Nachrichten zurckgelegt hat
            strecke = Math.sqrt (Math.pow (xalt - xneu, 2) + Math.pow (yalt - yneu, 2));

            //Abfangen von Werten die zu sehr abweichen --> nichts machen
            if (Math.abs (xball) < 10 || Math.abs (yball) < 10)
                {
                    //Berechne die Zielposition und gehe hin
                    z = strecke * 2;
                    ZielPosX = xneu + (xball * z);
                    ZielPosY = yneu + (yball * z);
                   
                    //Gehe zu der Zielposition
                    geheZu (ZielPosX, ZielPosY, p);
                }
             else
                {
                    
                }   
                                              
        }

        /**
         *
         * Berechnet, ob der Ball im Zustndigkeitsbereich des Spielers liegt
         *
         * @param xRadius  Abstand in X-Richtung von der Sollposition des Spielers welcher den Zustndigkeitsbereich definiert
         * @param yRadius  Abstand in Y-Richtung von der Sollposition des Spielers welcher den Zustndigkeitsbereich definiert
         * @return  true or false
         */
        public boolean ballinarea(double xRadius, double yRadius)
        {
            if ((Math.abs(weltmodell.ball.getXAbs() - weltmodell.Sollpos.getXAbs()) < xRadius) && (Math.abs(weltmodell.ball.getYAbs() - weltmodell.Sollpos.getYAbs()) < yRadius))
                return true;
            else
                return false;
        }

        /**
         *
         * Berechnet mit Hilfe der eigenen absoluten Position die Entfernung zu dem bergebenen Objekt
         *
         * @param objekt bergebenes Objekt vom Typ <code>ObjectPerception</code>
         * @return       Entfernung zu dem bergebenen Objekt
         */
        public double calcDistance(ObjectPerception objekt)
        {
            double distance = Math.sqrt(Math.pow(weltmodell.xAktAbs - objekt.getXAbs(), 2) + Math.pow(weltmodell.yAktAbs - objekt.getYAbs(), 2));
            return distance;
        }
        
        public double calcDistance(double Xpos,double Ypos)
        {
            double distance = Math.sqrt(Math.pow(weltmodell.xAktAbs - Xpos, 2) + Math.pow(weltmodell.yAktAbs - Ypos, 2));
            return distance;
        }


        //
        /**
         *
         * Gibt den Winkel zum Objekt relativ zum Blickwinkel des Spielers zurck
         *
         * @param objekt  Objekt vom Typ <code>ObjectPerception</code> zu dem der Winkel berechnet wird
         * @return        Winkel zum Objekt relativ zum Blickwinkel des Spielers
         */
        public double calcDirection(ObjectPerception objekt)
        {
            double aktangle;                    //Aktueller Blickwinkel
            double direction=0;                 //Winkel zur gewnschten Position relativ zum Blickwinkel
            double deltax;                      //Differenz der x-Werte
            double deltay;                      //Differenz der y-Werte
            double xPosAbs = weltmodell.xAktAbs;//Absolute x-Position des Spielers
            double yPosAbs = weltmodell.yAktAbs;//Absolute y-Position des Spielers
            double xPos, yPos;


            try{
                xPos = objekt.getXAbs();
            }
            catch (Exception e) {
                xPos=0;
            }

            try{
                yPos = objekt.getYAbs();
            }
            catch (Exception e2) {
                yPos=0;
            }

            //Holt den aktuellen Blickwinkel (in Grad)
            aktangle = weltmodell.getAngle();

            //Berechnung von deltax und deltay
            deltax = -xPosAbs + xPos;
            deltay = -yPosAbs + yPos;

            if (xPos >= weltmodell.xAktAbs){
                direction = -aktangle + atang(deltay/deltax);
            }
            if (xPos < weltmodell.xAktAbs ){
                direction = -aktangle -180 + atang(deltay/deltax);
            }

            if (direction > 180){direction = direction - 360;}
            if (direction < -180){direction = direction + 360;}
            return direction;
        }

        //

        /**
         *
         * Gibt den Winkel zum Objekt relativ zum Blickwinkel des Spielers zurck
         *
         * @param xPos  X-Koordinate des Objektes
         * @param yPos  Y-Koordinate des Objektes
         * @return      Winkel zum Objejt relativ zum Blickwinkel des Spielers
         */
        public double calcDirection(double xPos, double yPos)
        {
            ObjectPerception objekt = new ObjectPerception("xy-to-object", xPos, yPos);
            double direction = calcDirection(objekt);
            return direction;
        }

       /**
         *
         * Berechnet den Passempfaenger der vor dem Spieler steht und ihm am nhsten ist
         *
         * @return  Spieler dem der Ball zugepasst werden soll
         */
        public ObjectPerception PassEmpfaenger()
        {
            ObjectPerception Mitspieler = new ObjectPerception();
            Mitspieler.setDistance(200);
            ObjectPerception MitspielerNeu = new ObjectPerception();
            if(! weltmodell.mitspieler.isEmpty()){
                int i = weltmodell.mitspieler.indexOf(weltmodell.mitspieler.lastElement());

                while (i != 0)
                    {
                        MitspielerNeu = (ObjectPerception) weltmodell.mitspieler.get(i);
                        if (MitspielerNeu.getXAbs() > weltmodell.xAktAbs +10)
                            {
                                if (MitspielerNeu.getDistance() < Mitspieler.getDistance())
                                    Mitspieler = MitspielerNeu;
                            }
                        i--;
                    }
                if (Mitspieler.getDistance()==200)
                            Mitspieler=null;
            }
            else{Mitspieler=null;}
            return Mitspieler;
        }

        /**
         *
         * Berechnet <code>PosX</code> bei der der Spieler nicht mehr im Abseits steht
         *
         * @return  X-Koordinate zu der der Spieler hinlaufen soll, damirt er nichtmehr im Abseits steht
         */
        public double abseitsPosX()
        {
            double PosX2=0;
            double PosX1=0;
            int i = 0;

            if(! weltmodell.gegenspieler.isEmpty()){

                while (i < weltmodell.gegenspieler.indexOf(weltmodell.gegenspieler.lastElement()) +1)
                    {
                        ObjectPerception Gegenspieler = new ObjectPerception();
                        Gegenspieler = (ObjectPerception) weltmodell.gegenspieler.get(i);
                        if (Gegenspieler.getTime() >= weltmodell.time-2)
                        {
                        if (Gegenspieler.getXAbs() > PosX2)
                            {
                                if (Gegenspieler.getXAbs() > PosX1)
                                    {
                                    PosX2 = PosX1;
                                    PosX1 = Gegenspieler.getXAbs();
                                    }
                                else PosX2 = Gegenspieler.getXAbs();
                            }
                        }
                        i=i+1;
                    }
             }
             else {PosX2 = 20;}
             
             if (PosX2<10){PosX2=10;}
             
             return PosX2 - 5;

        }

       //Grundfunktionen:
       /**
         * Fngt den Ball
         *
         * @param direction Die Richtung "in die gefangen wird"
         */
        public void catchBall(double direction)
        {
            communication.send(new String("(catch " + direction +")"));
        }

        /**
         * Ruft einen String ber das Feld
         *
         * @param text Der String der gerufen wird
         */
        public void say(String text)
        {
            communication.send(new String("(say " + text +")"));
        }

        /**
         * Fhrt Laufimpuls aus in Blickrichtung
         *
         * @param power Strke des Laufimpulses
         */
        public void dash(double power)
        {
            communication.send(new String("(dash " + power + ")"));
        }

        /**
         * Dreht den Kopf um einen bestimmten Winkel
         *
         * @param angle Winkel um den der Kopf gedreht wird
         */
        public void turn_neck(double angle)
        {
            communication.send(new String("(turn_neck " + angle + ")"));
        }


        /**
         * Kickt den Ball
         *
         * @param power  "Hrte" des Schuses
         * @param direction Richtung des Schusses
         */
        public void kick(double power, double direction)
        {
            communication.send(new String("(kick " + power + " " + direction + ")"));
        }

        /**
         * Setzt den Spieler auf eine bestimmte Position auf dem Spielfeld
         *
         * @param x X-Koordinate der Position
         * @param y Y-Koordinate der Position
         */
        public void move(double x, double y)
        {
            communication.send(new String("(move " + x + " " + y + ")"));
        }

        /**
         * Dreht den Spieler um einen bestimmten Winkel
         *
         *
         * @param moment Winkel um den sich der Spieler drehen soll
         */
        public void turn(double moment)
        {
            communication.send(new String("(turn " + moment + ")"));
        }

        /**
         * Berechnet den arctan des bergebenen Wertes und gibt ihn in Grad zurck
         *
         *
         * @param de Wert dessen arctan berechnet werden soll
         * @return   arctan des bergebenen Werts in Grad
         */
        public double atang (double de){
             double hilfe = Math.atan(de);
          hilfe= hilfe * 180 / Math.PI;
          return hilfe;
       }
}
