/**
 * Die Klasse <code>Stuermer</code> erbt von der Klasse <code>Spieler</code>.
 * In der Klasse <code>Stuermer</code> steht, wie sich der Stuermer in bestimmten Situationen Verhalten soll.
 * Sie ist also die Intelligenz bzw. KI der Stuermer.
 *
 * @version <br>3.0 Copyright 2002 by Team 18<br><br>
 * @author
 * <ul>
 * <li><a href="mailto:matze@gmx.net">Matthias Kunze</a></li>
 * <li><a href="mailto:holger.roessig@gmx.de">Holger Rssig</a></li>
 * <li><a href="mailto:bstrauss@gmx.de">Bastian Strauss</a></li>
 * </ul>
 * @since <br>JDK v 1.3.1<br><br>
 * @see <br> Communication.java
 * @see Weltmodell.java <br><br>
 */

public class Stuermer extends Spieler
{
    private double xStart, yStart;        //Position des Spielers vor dem Anpfiff
    private int x=0;
    private int ab=0;
    private boolean drehen=false;
    /**
     * Konstruktor von <code>Stuermer</code>
     *
     * @param teamname    Teamname
     * @param servername  Name des Servers
     * @param serverport  Port des Servers
     * @param xStart      X-Startkoordinate
     * @param yStart      Y-Startkoordinate
     */
    public Stuermer(String teamname, String servername, int serverport, double xStart, double yStart)
    {
        this.xStart = xStart;
        this.yStart = yStart;
        this.teamname = teamname;
        this.communication = new Communication(servername, serverport);
        this.weltmodell = new Weltmodell(communication);
        communication.weltmodell = weltmodell;
        weltmodell.xStart = xStart;
        weltmodell.yStart = yStart;
        weltmodell.teamname = teamname;
        weltmodell.Sollpos.setXAbs(xStart+35);
        weltmodell.Sollpos.setYAbs(yStart);
    }

    /**
     * Startet den <code>Stuermer</code>
     *
     */
    public void run()
    {
        init(teamname);
        weltmodell.sensor.receive();
        status = CONNECTED;
        play();
    }
    /**
     * Die Taktik des <code>Stuermer</code>'s
     *
     */
    private void play()
    {
    while(true)
        {
            switch (status) {
            case NOT_CONNECTED:
                init(teamname);
                status = CONNECTED;
                break;
            case CONNECTED:
                if (weltmodell.side==1)
                {
                    move(xStart-10, yStart);
                    turn(180);
                }
                else
                    move(xStart, yStart);
                    weltmodell.sensor.receive();
                    status = PLAYING;
                break;
            case PLAYING:
                while(weltmodell.seezeitpunkt==false){weltmodell.sensor.receive();}
                weltmodell.seezeitpunkt=false;

                if (weltmodell.NormalPlay)  // Das Spiel laeuft
                {
                    drehen=false;
                    // Wenn die Ballinformation zu alt ist; Ball suchen
                    if (weltmodell.ball.getTime() == -1 || weltmodell.time > (weltmodell.ball.getTime()+5)){
                        turn(90);
                    }
                    // Wenn nah genug am Ball; etwas mit dem Ball machen
                    else if (weltmodell.ball.getDistance() < reichweite)
                     {
                        reichweite = defaultReichweite;
                        // Nah genug am Tor; dann aufs Tor schiessen
                        if ((weltmodell.goalR.getDistance() < 30)){
                            goalkick(100);
                          }
                         // Nicht nah genug am Tor; Passsempfaenger suchen
                         else
                         {
                            ObjectPerception PassEmpfaenger = PassEmpfaenger();
                            // Wenn kein Passempfaenger gefunden, Ball nach vorne bolzen
                            if (PassEmpfaenger == null){
                                goalkick(100);
                            }
                            // Wenn Passempfaenger gefunden, zu ihm passen
                            else{
                                pass(PassEmpfaenger,calcDistance(PassEmpfaenger) *3 );  
                            }
                         }
                     }
                     // Wenn der Ball im Zustaendigkeitsbereich ist, zu ihm hinlaufen
                    else if (ballinarea(18,12) || weltmodell.ball.getDistance() < 5)
                     {
                         if (weltmodell.ball.getDistance() > 2){
                             reichweite = 1.9;
                             geheZuBall(100);
                         }
                         else if (weltmodell.ball.getDistance() > 1){
                             reichweite = defaultReichweite;     
                             geheZuBall(65);
                         }
                         else
                             geheZuBall(40);
                     }
                      // Wenn der Ball nicht im Zustaendigkeitsbereich ist,
                      // Zur Sollposition laufen und dabei aufs Abseits achten
                      else
                     {
                         double abseitsPosX = abseitsPosX();
                         if (abseitsPosX < weltmodell.Sollpos.getXAbs() && weltmodell.ball.getXAbs()<weltmodell.xAktAbs)
                             {   ab++;                                
                                 if (ab>1){
                                    geheZu(abseitsPosX, weltmodell.Sollpos.getYAbs(),100);
                                    if (ab>5)
                                        {
                                        turn(calcDirection(52.5,0));
                                        ab=0;
                                        }
                                }
                             }
                         else
                             {
                             if (calcDistance(weltmodell.Sollpos) > 10)
                                 geheZu(weltmodell.Sollpos,100);
                             else if (calcDistance(weltmodell.Sollpos) > 2)
                                   geheZu(weltmodell.Sollpos,20);
                               }
                      }
                    }
                    

                // Das Spiel ist unterbrochen
                else
                {
                    // Der Gegner hat ein Tor erzielt
                    // Spieler zurcksetzen
                    if(weltmodell.opponentGoal)
                    {
                        move(xStart, yStart);
                        if (weltmodell.side==1)
                            turn(180);
                        weltmodell.opponentGoal = false;
                    }

                    // Wir haben ein Tor erzielt
                    // Spieler zurcksetzen
                    if (weltmodell.ourGoal)
                    {
                    if (weltmodell.side==0) 
                        {   
                        if(yStart==-24)
                            move(-24,13);   
                        else if(yStart==0)
                            move(-27,14); 
                        else if(yStart==24)
                            move(-29,15);
                        }
                    else
                        {         
                        if(yStart==-24)
                            move(-24,-13);   
                        else if(yStart==0)
                            move(-27,-14); 
                        else if(yStart==24)
                            move(-29,-15);  
                        }      
                        for (int x=0; x<25; x++)
                            turn(8);                                        
                        move(xStart-10, yStart);
                        weltmodell.ourGoal = false;
                    }

                    // Wir haben Anstoss
                    if(weltmodell.ourKickOff)
                    {
                        if (weltmodell.yAktAbs < 5 && weltmodell.yAktAbs > -5){
                              if ((weltmodell.ball.getDistance() < defaultReichweite) & (weltmodell.ball.getTime() > -1))
                               {
                                   if (drehen==false){ 
                                       for (int x=0; x<45; x++){turn(8);}
                                       drehen=true;   
                                   }
                                   turn( - weltmodell.ball.getDirection());
                                   pass(35,24,100);
                               }
                               else
                               {
                               if (weltmodell.ball.getTime() == -1 || weltmodell.time > (weltmodell.ball.getTime()+8))
                                  turn(45);
                                  else
                                  {
                                       if (weltmodell.ball.getDistance() > 1)
                                             geheZuBall(50);
                                         else
                                             geheZuBall(25);
                                  }
                               }
                        }
                        else {geheZu(weltmodell.Sollpos,100);}
                    }

                    // Der Gegner hat Anstoss
                    if(weltmodell.opponentKickOff)
                    {
                        geheZu(weltmodell.Sollpos,100);
                    }

                    // Der Gegner bzw. unser Torwart hat Ballbesitz
                    // Zurck auf Sollposition
                    if (weltmodell.opponentGoalieHasBall || weltmodell.ourGoalieHasBall
                        || weltmodell.opponentFreeKick || weltmodell.opponentKickIn
                        || weltmodell.opponentGoalKick || weltmodell.ourGoalKick
                        || weltmodell.opponentCornerKick)
                    {
                        double abseitsPosX = abseitsPosX();
                             if (abseitsPosX < weltmodell.Sollpos.getXAbs())
                             {
                                 geheZu(abseitsPosX, weltmodell.Sollpos.getYAbs(),100);
                             }

                             else
                             {
                                 if (calcDistance(weltmodell.Sollpos) > 10)
                                     geheZu(weltmodell.Sollpos,100);
                                 else if (calcDistance(weltmodell.Sollpos) > 2)
                                    turn(weltmodell.ball.getDirection());
                               }
                        }

                    // Wir haben Freistoss,Eckball oder Einwurf
                    // gleiches Verhalten wie beim "normalen" Spiel
                    if (weltmodell.ourFreeKick || weltmodell.ourKickIn || weltmodell.ourCornerKick)
                    {
                        weltmodell.NormalPlay=true;
                    }
                    
                    // Halbzeitpause. Nach dem Pausentee wieder auf die Startposition stellen.
                    if (weltmodell.beforeKickOff){
                            move(xStart, yStart);
                    }                    
                    
               }
          }
        }
    }
}
