/**
 * Die Klasse <code>Torwart</code> erbt von der Klasse <code>Spieler</code>.
 * In der Klasse <code>Torwart</code> steht, wie sich der Torwart in bestimmten Situationen Verhalten soll.
 * Sie ist also die Intelligenz bzw. KI des Torwarts.
 *
 * @version <br>3.0 Copyright 2002 by Team 18<br><br>
 * @author
 * <ul>
 * <li><a href="mailto:matze@gmx.net">Matthias Kunze</a></li>
 * <li><a href="mailto:holger.roessig@gmx.de">Holger Rssig</a></li>
 * <li><a href="mailto:bstrauss@gmx.de">Bastian Strauss</a></li>
 * </ul>
 * @since <br>JDK v 1.3.1<br><br>
 * @see <br> Communication.java
 * @see Weltmodell.java <br><br>
*/
public class Torwart extends Spieler
{
    private double xStart;
    private double yStart;

    /**
     * Konstruktor von <code>Torwart</code>
     *
     * @param teamname    Teamname
     * @param servername  Name des Servers
     * @param serverport  Port des Servers
     * @param xStart      X-Startkoordinate
     * @param yStart      Y-Startkoordinate
    */
    public Torwart(String teamname, String servername, int serverport, double xStart, double yStart)
    {
        this.xStart = xStart;
        this.yStart = yStart;
        this.teamname = teamname;
        this.communication = new Communication(servername, serverport);
        this.weltmodell = new Weltmodell(communication);
        communication.weltmodell = weltmodell;
        weltmodell.xStart = xStart;
        weltmodell.yStart = yStart;
        weltmodell.teamname = teamname;
        weltmodell.Sollpos.setXAbs(xStart);
        weltmodell.Sollpos.setYAbs(yStart);
    }

    /**
     * Meldet den <code>Torwart</code> am Server als <code>Torwart</code> an.
     *
     * @param teamname Teamname
     */
    public void init(String teamname)
    {
        communication.send(new String("(init " + teamname + " (version 7.0) (goalie) )"));
    }

    /**
     * Gibt die Spielrichtung an und startet den <code>Torwart</code>
     *
     */
    public void run()
    {
        init(teamname);
        weltmodell.sensor.receive();
        status = CONNECTED;
        play();
    }

    /**
     * Die Taktik des <code>Torwart</code>'s
     *
     */
    private void play()
    {
        while(true)
        {
            switch (status) {

            case NOT_CONNECTED:
                init(teamname);
                status = CONNECTED;
                break;

            case CONNECTED:
                    move(xStart, yStart);  // Spielrichtung von Links nach Rechts
                    if (weltmodell.side==1) // Spielrichtung von Rechts nach Links
                        turn(180);
                    status=PLAYING;
                    break;

            case PLAYING:
            while(weltmodell.seezeitpunkt==false){weltmodell.sensor.receive();}
                weltmodell.seezeitpunkt=false;
                // Das Spiel laeuft
                if (weltmodell.NormalPlay)
                {
                    if (weltmodell.time > (weltmodell.ball.getTime()+4)){
                        turn(90);}
                        
                    //Torwart ist nicht im Strafraum
                    else if ((weltmodell.yAktAbs < -20 || weltmodell.yAktAbs > 20) || weltmodell.xAktAbs > -36)
                    {
                        geheZu(weltmodell.Sollpos,100);
                    }
                        
                    //Ball ist im Strafraum ==> zum Ball laufen und fangen
                    else if (weltmodell.ball.getXAbs() < -36 && weltmodell.ball.getYAbs() > -20 && weltmodell.ball.getYAbs() < 20)
                    {
                        if (weltmodell.ball.getDistance() > 3){
                            geheZuBall(100);}
                        else
                            if (weltmodell.ball.getDistance() > 2){
                                geheZuBall(50);}
                            if (weltmodell.ball.getDistance() <= 2)
                            {
                                if (weltmodell.ball.getYAbs() > 0)
                                {
                                    catchBall(weltmodell.ball.getDirection());
                                    move(-35,-20);
                                }
                                if (weltmodell.ball.getYAbs() <= 0)
                                {
                                    catchBall(weltmodell.ball.getDirection());
                                    move(-35,20);
                                }
                                while (weltmodell.seezeitpunkt!=true)
                                {weltmodell.sensor.receive();}
                                weltmodell.seezeitpunkt = false;
                                goalkick(100);
                            }
                    }
                    
                    else    //Ball ist nicht im Strafraum
                    {
                        //Ball ist auf der Seite der Gegner
                        if (weltmodell.ball.getXAbs() > -5)
                        {
                            if (calcDistance(weltmodell.Sollpos) > 10){
                                geheZuDirekt(weltmodell.Sollpos,100);}
                            else
                                if (calcDistance(weltmodell.Sollpos) > 5){
                                    geheZuDirekt(weltmodell.Sollpos,50);}
                            else
                                if (calcDistance(weltmodell.Sollpos) < 1.5){}
                        }
                        //Ball ist auf unserer Seite
                        else
                        {
                            if (calcDistance(weltmodell.Sollpos) > 10){
                                geheZu(weltmodell.Sollpos,100);}
                            else
                                if (calcDistance(weltmodell.Sollpos) > 5){
                                    geheZu(weltmodell.Sollpos,50);}
                            else
                                if (calcDistance(weltmodell.Sollpos) < 1.5){}
                        }
                    }
                }

                else    // Das Spiel ist unterbrochen
                {
                    // Wir haben Anstoss
                    if(weltmodell.ourKickOff){
                       move(xStart, yStart); }

                    // Der Gegner hat Anstoss
                    if(weltmodell.opponentKickOff){
                         move(xStart, yStart);  }

                    // Der Gegner hat ein Tor erzielt
                    if(weltmodell.opponentGoal){
                            move(xStart, yStart);}

                    // Wir haben ein Tor erzielt
                    if (weltmodell.ourGoal){
                        move(-30,0); 
                        for (int x=0; x<25; x++)
                            turn(8); 
                        move(xStart, yStart);
                        weltmodell.ourGoal = false;
                        }

                    // Wir haben Absto
                    if (weltmodell.ourGoalKick)
                    {
                        if (weltmodell.ball.getTime() == -1 || weltmodell.time > (weltmodell.ball.getTime()+4)){
                            turn(90);}                        
                        else if (calcDistance(weltmodell.ball) > 5)
                        {
                            geheZuBall(100);
                        }
                        else if (calcDistance(weltmodell.ball) > 3)
                        {
                            geheZuBall(60);
                        }
                        else if (calcDistance(weltmodell.ball) > 0.4)
                        {                                 
                            geheZuBall(25);
                        }
                        else if (calcDistance(weltmodell.ball) < 0.5)
                        {
                            goalkick(100);
                        }
                    }

                    // Wir haben Freistoss oder Einwurf
                    // Der Gegener hat Freistoss oder Einwurf
                    // gleiches Verhalten wie beim "normalen" Spiel
                    if (weltmodell.ourFreeKick || weltmodell.ourKickIn || weltmodell.opponentFreeKick || weltmodell.opponentKickIn)
                    {
                        //Ball ist auf der Seite der Gegner
                        if (weltmodell.ball.getXAbs() > -5)
                        {
                            if (calcDistance(weltmodell.Sollpos) > 10){
                                geheZuDirekt(weltmodell.Sollpos,100);}
                            else
                                if (calcDistance(weltmodell.Sollpos) > 5){
                                    geheZuDirekt(weltmodell.Sollpos,50);}
                                else
                                    if (calcDistance(weltmodell.Sollpos) < 1.5){}
                         }
                         //Ball ist auf unserer Seite
                         else
                         {
                             if (calcDistance(weltmodell.Sollpos) > 10){
                                 geheZu(weltmodell.Sollpos,100);}
                             else
                                 if (calcDistance(weltmodell.Sollpos) > 5){
                                     geheZu(weltmodell.Sollpos,50);}
                                 else
                                     if (calcDistance(weltmodell.Sollpos) < 1.5){}
                          }
                    }

                    // Der Gegner hat Eckball, gehe zum Kurzen-Eck
                    if (weltmodell.opponentCornerKick)
                    {
                        if(weltmodell.ball.getYAbs() < 0)
                        {
                            if (calcDistance(weltmodell.f_g_l_t) > 10){
                                geheZu(weltmodell.f_g_l_t,100);}
                            else
                                if (calcDistance(weltmodell.f_g_l_t) > 1.5){
                                    geheZu(weltmodell.f_g_l_t,20);}
                        }
                        else
                        {
                            if (calcDistance(weltmodell.f_g_l_b) > 10){
                                geheZu(weltmodell.f_g_l_b,100);}
                            else
                                if (calcDistance(weltmodell.f_g_l_b) > 1.5){
                                    geheZu(weltmodell.f_g_l_b,20);}
                        }
                    }

                    // Wir haben Eckball
                    if (weltmodell.ourCornerKick){
                        weltmodell.NormalPlay = true;}
                }
            }
        }
    }
}
