/**
 * In der Klasse <code>AcousticEvent</code> werden alle Hear Nachrichten analysiert,
 * interpretiert und im Weltmodell abgespeichert.
 *
 * @version <br>3.0 Copyright 2002 by Team 18<br><br>
 * @author
 * <ul>
 * <li><a href="mailto:heiner.litz@blauton.com">Heiner Litz</a></li>
 * </ul>
 * @since <br>JDK v 1.3.1<br><br>
 * @see <br> Weltmodell.java <br><br>
 */

import java.lang.*;
import java.util.*;

public class VisualEvent extends Event
{
    private Weltmodell weltmodell;

    /**
     * Sucht eine Feldlinie um die Positionsbestimmung zu ermglichen
     *
     * @return Feldlinie mit aktuellem Zeitstempel
     */
    public ObjectPerception actualLine(){
        if(weltmodell.lineL.getTime()==weltmodell.time && weltmodell.time!=0){
            return weltmodell.lineL;
        }
        else if(weltmodell.lineR.getTime()==weltmodell.time && weltmodell.time!=0){
            return weltmodell.lineR;
        }
        else if(weltmodell.lineT.getTime()==weltmodell.time && weltmodell.time!=0){
            return weltmodell.lineT;
        }
        else if(weltmodell.lineB.getTime()==weltmodell.time && weltmodell.time!=0){
            return weltmodell.lineB;
        }
        else{
            return null;
        }
    }

    /**
     * Berechnet den Blickwinkel des Spielers
     *
     * @return Blickwinkel
     */
    public double viewingAngle(){
        double angle=0;

        //welche Feldlinie wurde zuletzt gesehen?
        ObjectPerception l = actualLine();
        if (l!=null){
            switch (l.getName().charAt(4) ){
                //Blickwinkel wird mit Hilfe der oberen Linie berechnet
                case 'T':
                    if (l.getDirection() > 0){
                        angle = - l.getDirection();
                    }
                    else {
                        angle = -180 - l.getDirection();
                    }
                    break;

                //Blickwinkel wird mit Hilfe der unteren Linie berechnet
                case 'B':
                    if (l.getDirection() > 0){
                        angle = 180 - l.getDirection();
                    }
                    else {
                        angle =  - l.getDirection();
                    }
                    break;

                //Blickwinkel wird mit Hilfe der rechten Linie berechnet
                case 'R':
                    if (l.getDirection() > 0){
                        angle = 90 - l.getDirection();
                    }
                    else {
                        angle = - (90 + l.getDirection());
                    }
                    break;

                //Blickwinkel wird mit Hilfe der linken Linie berechnet
                case 'L':
                    if (l.getDirection() > 0){
                        angle = - (90 + l.getDirection() );
                    }
                    else {
                        angle = 90 - l.getDirection();
                    }
                    break;
            }
            //Falls von links nach rechts gespielt wird muss der Winkel invertiert werden
            if (weltmodell.side==1){angle=angle-180;}

            weltmodell.setAngle(angle);
            return angle;
        }
        //Falls keine Linie gefunden wurde wird ein Pseudo Winkel zurckgegeben
        else {return 333;}
    }

    /**
     * Berechnet die absoluten Koordinaten des bergebenen Objektes
     *
     * @param Spieler oder Ball
     */
    public void getAbs(ObjectPerception o){
        double dis = o.getDistance();
        
        //Berechnet aus der eigenen Absoluten Position und dem Winkel und der Distanz zum
        //bergebenen Objekt die Absolute Position des Objekts
        o.setXAbs(weltmodell.xAktAbs + dis * Math.cos(
            Math.toRadians(o.getDirection() + weltmodell.getAngle() )
            ));
        o.setYAbs(weltmodell.yAktAbs + dis * Math.sin(
            Math.toRadians(o.getDirection() + weltmodell.getAngle())
        ));
    }


    /**
     * Sucht diejenige Flagge welche dem Spieler am nchsten ist, mit aktuellem Zeitstempel
     *
     * @return nchstgelegenes Objekt
     */
    public ObjectPerception findNextObject(){
        int i =0;
        ObjectPerception nearest = new ObjectPerception("Objekt1",0,0);
        double dist=100;
        while(i<53){
            //Falls der Zeitstempel aktuell und die Distanz zur Flagge kleiner als die Distanz von
            //nearest ist, speichere diese Flagge in nearest
            if((weltmodell.flagArray[i]).getDistance() < dist && (weltmodell.flagArray[i]).getTime() == weltmodell.time){
                nearest = weltmodell.flagArray[i];
                dist = nearest.getDistance();
            }
            i++;
        }

    return nearest;
    }

    /**
     * Berechnet die absolute Position des Spielers und trgt diese ins Weltmodell ein
     */
    private void getAbsPlayerPos(){
        double dx =0;
        double dy =0;
        double angle = viewingAngle();
        if (weltmodell.side==1){angle=angle+180;}
        //Falls viewingAngle() keinen Fehlerwert zurckgegeben hat
        if (angle!=333){
            ObjectPerception o = findNextObject();
            double direction = o.getDirection();
            double distance = o.getDistance();
            
            //Berechnet aus dem Winkel und der Distanz des Spielers zu einem statischen Object, die eigene Absolute Position
            weltmodell.xAktAbs = o.getXAbs() - ((distance) * (java.lang.Math.cos(java.lang.Math.toRadians(angle + direction)))  );
            weltmodell.yAktAbs = o.getYAbs() - ((distance) * (java.lang.Math.sin(java.lang.Math.toRadians(angle + direction)))  );
            //Falls die Spielrichtung von rechts->links ist muss die Absolute Position invertiert werden
            if (weltmodell.side==1){
                weltmodell.xAktAbs = weltmodell.xAktAbs * (-1);
                weltmodell.yAktAbs = weltmodell.yAktAbs * (-1);
            }      
        }
        else {
            //Falls kein statisches Objekt gefunden wurde bernehme default Werte
            weltmodell.xAktAbs = weltmodell.xStart;
            weltmodell.yAktAbs = weltmodell.yStart;
        }
    }

    /**
     * Erstellt eine neue Instanz des Typs <code>VisualEvent</code> und verwendet einen Tokanizer,
     * um die bergebene Nachricht zu unterteilen und die relevanten Informationen zu extrahieren.
     * Diese werden interpretiert und in den Variablen des Weltmodells gespeichert.
     *
     * @param hear message  Nachricht des Servers
     * @param weltmodell  Weltmodell des Spielers
    */
    public VisualEvent(String message, Weltmodell weltmodell)
    {
        this.weltmodell = weltmodell;
        // benutzt Klammern (,) als Begrenzungszeichen fr die Zeichenfolge
        StringTokenizer tokenizer = new StringTokenizer(message, "()");

        int time = Integer.parseInt(tokenizer.nextToken().substring(4).trim());
        weltmodell.time=time;

        while (tokenizer.hasMoreTokens())
        {
            String name=tokenizer.nextToken();
            // manchmal werden ankommende Nachrichten gesehen, ohne das Opbjekte folgen
            if (!name.equals(" "))
            {
                
                if(tokenizer.hasMoreTokens()){
                
                // erhalte Entfernung und Richtung
                String data = tokenizer.nextToken();
                
                if(tokenizer.hasMoreTokens()){
                
                // wechsel zum nchsten Objekt
                tokenizer.nextToken();

                switch (name.charAt(0)) {
                // um verschiedene Typen zu analysieren, wird ein Tokenizer mit Leerzeichen als Begrenzungszeichen benutzt
                
                //Es wurde ein Ball gesehen
                case 'b':       
                    weltmodell.seeball = true;
                    StringTokenizer st   = new StringTokenizer(data, " ");
                    //Speichert die alte Ball Position
                    weltmodell.alterBall.setTime(weltmodell.ball.getTime());
                    weltmodell.alterBall.setName(weltmodell.ball.getName());
                    weltmodell.alterBall.setDistance(weltmodell.ball.getDistance());
                    weltmodell.alterBall.setDirection(weltmodell.ball.getDirection());
                    weltmodell.alterBall.setXAbs(weltmodell.ball.getXAbs());
                    weltmodell.alterBall.setYAbs(weltmodell.ball.getYAbs());
                    
                    // speichert die Daten im Weltmodell
                    weltmodell.ball.setTime(time);
                    weltmodell.ball.setDistance(Double.valueOf(st.nextToken()).doubleValue()); //Double.parseDouble(st.nextToken());
                    weltmodell.ball.setDirection(Double.valueOf(st.nextToken()).doubleValue()); //Double.parseDouble(st.nextToken();
                    break;

                //Es wurde ein Tor gesehen
                case 'g':
                    StringTokenizer st1 = new StringTokenizer(data, " ");
                    double distance1  = Double.valueOf(st1.nextToken()).doubleValue(); //Double.parseDouble(st.nextToken());
                    double direction1 = Double.valueOf(st1.nextToken()).doubleValue(); //Double.parseDouble(st.nextToken());

                    // speichert die Daten im linken oder rechten Tor
                    if (name.equals("g r")) {
                        weltmodell.goalR.setTime(time);
                        weltmodell.goalR.setDistance(distance1);
                        weltmodell.goalR.setDirection(direction1);
                    }
                    if (name.equals("g l")) {
                        weltmodell.goalL.setTime(time);
                        weltmodell.goalL.setDistance(distance1);
                        weltmodell.goalL.setDirection(direction1);
                    }
                    break;

                //Es wurde ein Spieler gesehen
                case 'p':
                    //Falls der Spieler zu weit entfernt ist, wird fr team und nummer ?? gesetzt.
                    String team ="??";
                    String nummer="??";
                    StringTokenizer st4 = new StringTokenizer(data, " ");
                    double distance = Double.valueOf(st4.nextToken()).doubleValue();
                    double direction= Double.valueOf(st4.nextToken()).doubleValue();
                    StringTokenizer st5 = new StringTokenizer(name, " ");
                    st5.nextToken();
                    if(st5.hasMoreTokens()){
                        //In welchem Team spielt der Spieler?
                        team = st5.nextToken();         
                        if(st5.hasMoreTokens()){
                            //Die Rckennummer des Spielers = name
                            nummer = st5.nextToken();   
                        }
                    }
                    ObjectPerception spieler = new ObjectPerception(time, nummer, distance, direction);
                    //Absolute Position des gesehenen Spielers ermitteln
                    getAbs(spieler);                    

                    //Abfrage ob der gesehene Spieler Mitspieler oder Gegenspieler ist.
                    //Einfgen des oben erstellten Object in die entsprechende Liste (Vector)

                    if(team.substring(1, team.length()-1).equals(weltmodell.teamname) && weltmodell.time!=3000 && weltmodell.time!=0){weltmodell.addMitspieler(spieler);}

                    else if (weltmodell.time!=3000 && weltmodell.time!=0){ weltmodell.addGegenspieler(spieler);}

                    //zeitlich veraltete Mitspieler entfernen
                    while (!weltmodell.mitspieler.isEmpty() && (((ObjectPerception)weltmodell.mitspieler.firstElement()).getTime() < weltmodell.time-2)){
                            weltmodell.mitspieler.removeElementAt(0);
                    }

                    while (!weltmodell.gegenspieler.isEmpty() && (((ObjectPerception)weltmodell.gegenspieler.firstElement()).getTime() < weltmodell.time-8)){
                            weltmodell.gegenspieler.removeElementAt(0);
                    }

                    break;
                
                //Es wurde eine Flagge gesehen
                case 'f':
                    StringTokenizer st2   = new StringTokenizer(data, " ");
                    double distance2  = Double.valueOf(st2.nextToken()).doubleValue(); //Double.parseDouble(st.nextToken());
                    double direction2 = Double.valueOf(st2.nextToken()).doubleValue(); //Double.parseDouble(st.nextToken());

                    // erstellt eine neue Objektinformation und speichert diese im FlagArray
                    //Die Tokens werden verndert, um sie verwenden zu knnen: f p l b => f_p_l_b
                    char leer = ' ';
                    char tief = '_';
                    String s = name.replace(leer,tief);

                    for (int x=0; x<53 ; x++){
                        if ((weltmodell.flagArray[x]).getName().equals(s)){
                            (weltmodell.flagArray[x]).setDistance(distance2);
                            (weltmodell.flagArray[x]).setDirection(direction2);
                            (weltmodell.flagArray[x]).setTime(time);
                        }
                    }
                    break;
                
                //Es wurde eine Linie gesehen
                case 'l':
                    StringTokenizer st3   = new StringTokenizer(data, " ");
                    double distance3  = Double.valueOf(st3.nextToken()).doubleValue(); //Double.parseDouble(st.nextToken());
                    double direction3 = Double.valueOf(st3.nextToken()).doubleValue(); //Double.parseDouble(st.nextToken());

                    //Prfen ob die gesehene Linie oben, unten, rechts oder links ist.
                    switch(name.charAt(2)){
                        case 't':
                        weltmodell.lineT.setTime(time);
                        weltmodell.lineT.setDistance(distance3);
                        weltmodell.lineT.setDirection(direction3);
                        break;

                        case 'b':
                        weltmodell.lineB.setTime(time);
                        weltmodell.lineB.setDistance(distance3);
                        weltmodell.lineB.setDirection(direction3);
                        break;

                        case 'r':
                        weltmodell.lineR.setTime(time);
                        weltmodell.lineR.setDistance(distance3);
                        weltmodell.lineR.setDirection(direction3);
                        break;

                        case 'l':
                        weltmodell.lineL.setTime(time);
                        weltmodell.lineL.setDistance(distance3);
                        weltmodell.lineL.setDirection(direction3);
                        break;
                    }

                }}}
            }
        }
        getAbsPlayerPos();
        //absolute Ballposition berechnen
        if (weltmodell.seeball){
            getAbs(weltmodell.ball);
            weltmodell.seeball = false;   
        }
    }
}

